@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,6 +52,18 @@ G_DEFINE_QUARK (g-io-error-quark, g_io_error)
  * handled (but note that future GLib releases may return a more
  * specific value instead).
  *
+ * As %errno is global and may be modified by intermediate function
+ * calls, you should save its value as soon as the call which sets it
+ * returns:
+ * |[
+ *   int saved_errno;
+ *
+ *   ret = read (blah);
+ *   saved_errno = errno;
+ *
+ *   g_io_error_from_errno (saved_errno);
+ * ]|
+ *
  * Returns: #GIOErrorEnum value for the given errno.h error number.
  **/
 GIOErrorEnum
@@ -95,12 +107,24 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
+#ifdef ENXIO
+    case ENXIO:
+      return G_IO_ERROR_NOT_REGULAR_FILE;
+      break;
+#endif
+
 #ifdef EROFS
     case EROFS:
       return G_IO_ERROR_READ_ONLY;
       break;
 #endif
 
+#ifdef EMLINK
+    case EMLINK:
+      return G_IO_ERROR_TOO_MANY_LINKS;
+      break;
+#endif
+
 #ifdef ELOOP
     case ELOOP:
       return G_IO_ERROR_TOO_MANY_LINKS;
@@ -193,6 +217,12 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
+#ifdef ETXTBSY
+    case ETXTBSY:
+      return G_IO_ERROR_BUSY;
+      break;
+#endif
+
 #ifdef EWOULDBLOCK
     case EWOULDBLOCK:
       return G_IO_ERROR_WOULD_BLOCK;
@@ -260,6 +290,12 @@ g_io_error_from_errno (gint err_no)
       break;
 #endif
 
+#ifdef ENOTSOCK
+    case ENOTSOCK:
+      return G_IO_ERROR_INVALID_ARGUMENT;
+      break;
+#endif
+
     default:
       return G_IO_ERROR_FAILED;
       break;
@@ -325,6 +361,8 @@ g_io_error_from_win32_error (gint error_code)
       return G_IO_ERROR_NOT_SUPPORTED;
 
     case WSAECONNRESET:
+    case WSAENETRESET:
+    case WSAESHUTDOWN:
       return G_IO_ERROR_CONNECTION_CLOSED;
 
     case WSAEHOSTUNREACH:
