@@ -267,7 +267,7 @@ remove_network (GNetworkMonitorNetlink *nl,
   if (nl->priv->dump_networks)
     {
       GInetAddressMask **dump_networks = (GInetAddressMask **)nl->priv->dump_networks->pdata;
-      int i;
+      guint i;
 
       for (i = 0; i < nl->priv->dump_networks->len; i++)
         {
@@ -392,7 +392,7 @@ read_netlink_messages (GNetworkMonitorNetlink  *nl,
               if (!nl->priv->dump_networks &&
                   rtmsg->rtm_family == AF_INET6 &&
                   rtmsg->rtm_dst_len != 0 &&
-                  UNALIGNED_IN6_IS_ADDR_MC_LINKLOCAL (dest))
+                  (dest && UNALIGNED_IN6_IS_ADDR_MC_LINKLOCAL (dest)))
                 continue;
 
               if (msg->nlmsg_type == RTM_NEWROUTE)
