@@ -180,17 +180,18 @@ elf_foreach_resource_section (Elf             *elf,
                               SectionCallback  callback,
                               gpointer         data)
 {
+  int ret G_GNUC_UNUSED  /* when compiling with G_DISABLE_ASSERT */;
   size_t shstrndx, shnum;
   size_t scnidx;
   Elf_Scn *scn;
   GElf_Shdr *shdr, shdr_mem;
   const gchar *section_name;
 
-  elf_getshdrstrndx (elf, &shstrndx);
-  g_assert (shstrndx >= 0);
+  ret = elf_getshdrstrndx (elf, &shstrndx);
+  g_assert (ret == 0);
 
-  elf_getshdrnum (elf, &shnum);
-  g_assert (shnum >= 0);
+  ret = elf_getshdrnum (elf, &shnum);
+  g_assert (ret == 0);
 
   for (scnidx = 1; scnidx < shnum; scnidx++)
     {
@@ -476,8 +477,8 @@ static gint
 cmd_help (gboolean     requested,
           const gchar *command)
 {
-  const gchar *description;
-  const gchar *synopsis;
+  const gchar *description = NULL;
+  const gchar *synopsis = NULL;
   gchar *option;
   GString *string;
 
