@@ -32,8 +32,6 @@ G_BEGIN_DECLS
 
 typedef struct {
   GResolver parent_instance;
-
-  GThreadPool *thread_pool;
 } GThreadedResolver;
 
 typedef struct {
@@ -44,6 +42,19 @@ typedef struct {
 GLIB_AVAILABLE_IN_ALL
 GType g_threaded_resolver_get_type (void) G_GNUC_CONST;
 
+/* Used for a private test API */
+#ifdef G_OS_UNIX
+GLIB_AVAILABLE_IN_ALL
+GList *g_resolver_records_from_res_query (const gchar      *rrname,
+                                          gint              rrtype,
+                                          const guint8     *answer,
+                                          gssize            len,
+                                          gint              herr,
+                                          GError          **error);
+GLIB_AVAILABLE_IN_ALL
+gint g_resolver_record_type_to_rrtype (GResolverRecordType type);
+#endif
+
 G_END_DECLS
 
 #endif /* __G_RESOLVER_H__ */
