@@ -121,7 +121,7 @@ test_auth_on_timeout (gpointer user_data)
 {
   g_error ("Timeout waiting for client");
   g_assert_not_reached ();
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 
@@ -167,6 +167,7 @@ test_auth_mechanism (const gchar *allowed_client_mechanism,
   GMainLoop *loop;
   GThread *client_thread;
   TestAuthData data;
+  guint timeout_id;
 
   server = server_new_for_mechanism (allowed_server_mechanism);
 
@@ -177,7 +178,7 @@ test_auth_mechanism (const gchar *allowed_client_mechanism,
                     G_CALLBACK (test_auth_on_new_connection),
                     loop);
 
-  g_timeout_add_seconds (5, test_auth_on_timeout, NULL);
+  timeout_id = g_timeout_add_seconds (5, test_auth_on_timeout, NULL);
 
   data.allowed_client_mechanism = allowed_client_mechanism;
   data.allowed_server_mechanism = allowed_server_mechanism;
@@ -195,6 +196,7 @@ test_auth_mechanism (const gchar *allowed_client_mechanism,
   g_dbus_server_stop (server);
 
   g_thread_join (client_thread);
+  g_source_remove (timeout_id);
 
   while (g_main_context_iteration (NULL, FALSE));
   g_main_loop_unref (loop);
