@@ -21,7 +21,9 @@
  */
 
 /* We are testing some deprecated APIs here */
+#ifndef GLIB_DISABLE_DEPRECATION_WARNINGS
 #define GLIB_DISABLE_DEPRECATION_WARNINGS
+#endif
 
 #include <glib.h>
 
@@ -85,7 +87,7 @@ acquire (gint nr)
   if (!g_rec_mutex_trylock (&locks[nr]))
     {
       if (g_test_verbose ())
-        g_print ("thread %p going to block on lock %d\n", self, nr);
+        g_printerr ("thread %p going to block on lock %d\n", self, nr);
 
       g_rec_mutex_lock (&locks[nr]);
     }
@@ -100,7 +102,7 @@ acquire (gint nr)
   g_assert (owners[nr] == self);   /* hopefully this is still us... */
 
   if (g_test_verbose ())
-    g_print ("thread %p recursively taking lock %d\n", self, nr);
+    g_printerr ("thread %p recursively taking lock %d\n", self, nr);
 
   g_rec_mutex_lock (&locks[nr]);  /* we're recursive, after all */
 
