@@ -126,6 +126,8 @@ test_far_future_ready_time (void)
   n_fds = 0;
   n_fds = g_main_context_query (context, priority, &timeout_, NULL, n_fds);
 
+  g_assert_cmpint (n_fds, >=, 0);
+
   /* The true timeout in milliseconds doesn't fit into a gint. We definitely
    * don't want poll() to block forever:
    */
@@ -150,11 +152,13 @@ test_func (gpointer data)
 
   /* We accept 2 on the first iteration because _add_seconds() can
    * have an initial latency of 1 second, see its documentation.
+   *
+   * Allow up to 500ms leeway for rounding and scheduling.
    */
   if (count == 0)
-    g_assert (current_time / 1000000 - last_time / 1000000 <= 2);
+    g_assert_cmpint (current_time / 1000 - last_time / 1000, <=, 2500);
   else
-    g_assert (current_time / 1000000 - last_time / 1000000 == 1);
+    g_assert_cmpint (current_time / 1000 - last_time / 1000, <=, 1500);
 
   last_time = current_time;
   count++;
