@@ -30,6 +30,7 @@
  */
 #include "config.h"
 
+#include <glib.h>
 #include <stdio.h>
 #include <windows.h>
 
@@ -39,34 +40,38 @@
 #include <sys/cygwin.h>
 #endif
 
-static void
-set_error (const gchar *format,
-	   ...)
+static void G_GNUC_PRINTF (2, 3)
+set_error (GError      **error,
+           const gchar  *format,
+           ...)
 {
-  gchar *error;
+  gchar *win32_error;
   gchar *detail;
   gchar *message;
   va_list args;
 
-  error = g_win32_error_message (GetLastError ());
+  win32_error = g_win32_error_message (GetLastError ());
 
   va_start (args, format);
   detail = g_strdup_vprintf (format, args);
   va_end (args);
 
-  message = g_strconcat (detail, error, NULL);
+  message = g_strconcat (detail, win32_error, NULL);
 
   g_module_set_error (message);
+  g_set_error_literal (error, G_MODULE_ERROR, G_MODULE_ERROR_FAILED, message);
+
   g_free (message);
   g_free (detail);
-  g_free (error);
+  g_free (win32_error);
 }
 
 /* --- functions --- */
 static gpointer
 _g_module_open (const gchar *file_name,
 		gboolean     bind_lazy,
-		gboolean     bind_local)
+		gboolean     bind_local,
+                GError     **error)
 {
   HINSTANCE handle;
   wchar_t *wfilename;
@@ -83,7 +88,7 @@ _g_module_open (const gchar *file_name,
   /* suppress error dialog */
   success = SetThreadErrorMode (SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS, &old_mode);
   if (!success)
-    set_error ("");
+    set_error (error, "");
 
   /* When building for UWP, load app asset DLLs instead of filesystem DLLs.
    * Needs MSVC, Windows 8 and newer, and is only usable from apps. */
@@ -98,7 +103,7 @@ _g_module_open (const gchar *file_name,
   g_free (wfilename);
       
   if (!handle)
-    set_error ("'%s': ", file_name);
+    set_error (error, "'%s': ", file_name);
 
   return handle;
 }
@@ -117,7 +122,7 @@ _g_module_close (gpointer handle)
 {
   if (handle != null_module_handle)
     if (!FreeLibrary (handle))
-      set_error ("");
+      set_error (NULL, "");
 }
 
 static gpointer
@@ -131,7 +136,20 @@ find_in_any_module_using_toolhelp (const gchar *symbol_name)
   /* Under UWP, Module32Next and Module32First are not available since we're
    * not allowed to search in the address space of arbitrary loaded DLLs */
 #if !defined(G_WINAPI_ONLY_APP)
-  if ((snapshot = CreateToolhelp32Snapshot (TH32CS_SNAPMODULE, 0)) == (HANDLE) -1)
+  /* https://docs.microsoft.com/en-us/windows/win32/api/tlhelp32/nf-tlhelp32-createtoolhelp32snapshot#remarks
+   * If the function fails with ERROR_BAD_LENGTH, retry the function until it succeeds. */
+  while (TRUE)
+    {
+      snapshot = CreateToolhelp32Snapshot (TH32CS_SNAPMODULE, 0);
+      if (snapshot == INVALID_HANDLE_VALUE && GetLastError () == ERROR_BAD_LENGTH)
+        {
+          g_thread_yield ();
+          continue;
+        }
+      break;
+    }
+
+  if (snapshot == INVALID_HANDLE_VALUE)
     return NULL;
 
   me32.dwSize = sizeof (me32);
@@ -176,7 +194,7 @@ _g_module_symbol (gpointer     handle,
     p = GetProcAddress (handle, symbol_name);
 
   if (!p)
-    set_error ("");
+    set_error (NULL, "");
 
   return p;
 }
