@@ -80,11 +80,14 @@ test_object_set_quux (TestObject  *obj,
 static void
 test_object_finalize (GObject *gobject)
 {
-  g_free (((TestObject *) gobject)->baz);
+  TestObject *self = (TestObject *) gobject;
+
+  g_free (self->baz);
+  g_free (self->quux);
 
   /* When the ref_count of an object is zero it is still
    * possible to notify the property, but it should do
-   * nothing and silenty quit (bug #705570)
+   * nothing and silently quit (bug #705570)
    */
   g_object_notify (gobject, "foo");
   g_object_notify_by_pspec (gobject, properties[PROP_BAR]);
@@ -101,8 +104,7 @@ test_object_set_property (GObject *gobject,
   TestObject *tobj = (TestObject *) gobject;
 
   g_assert_cmpint (prop_id, !=, 0);
-  g_assert_cmpint (prop_id, !=, N_PROPERTIES);
-  g_assert (pspec == properties[prop_id]);
+  g_assert_true (prop_id < N_PROPERTIES && pspec == properties[prop_id]);
 
   switch (prop_id)
     {
@@ -136,8 +138,7 @@ test_object_get_property (GObject *gobject,
   TestObject *tobj = (TestObject *) gobject;
 
   g_assert_cmpint (prop_id, !=, 0);
-  g_assert_cmpint (prop_id, !=, N_PROPERTIES);
-  g_assert (pspec == properties[prop_id]);
+  g_assert_true (prop_id < N_PROPERTIES && pspec == properties[prop_id]);
 
   switch (prop_id)
     {
@@ -220,54 +221,54 @@ typedef struct {
 static void
 on_notify (GObject           *gobject,
            GParamSpec        *pspec,
-           TestNotifyClosure *clos)
+           TestNotifyClosure *closure)
 {
-  g_assert (clos->pspec == pspec);
-  g_assert_cmpstr (clos->name, ==, pspec->name);
-  clos->fired = TRUE;
+  g_assert (closure->pspec == pspec);
+  g_assert_cmpstr (closure->name, ==, pspec->name);
+  closure->fired = TRUE;
 }
 
 static void
 properties_notify (void)
 {
   TestObject *obj = g_object_new (test_object_get_type (), NULL);
-  TestNotifyClosure clos;
+  TestNotifyClosure closure;
 
   g_assert (properties[PROP_FOO] != NULL);
   g_assert (properties[PROP_QUUX] != NULL);
-  g_signal_connect (obj, "notify", G_CALLBACK (on_notify), &clos);
+  g_signal_connect (obj, "notify", G_CALLBACK (on_notify), &closure);
 
-  clos.name = "foo";
-  clos.pspec = properties[PROP_FOO];
+  closure.name = "foo";
+  closure.pspec = properties[PROP_FOO];
 
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "foo", 47, NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
-  clos.name = "baz";
-  clos.pspec = properties[PROP_BAZ];
+  closure.name = "baz";
+  closure.pspec = properties[PROP_BAZ];
 
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "baz", "something new", NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
   /* baz lacks explicit notify, so we will see this twice */
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "baz", "something new", NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
   /* quux on the other hand, ... */
-  clos.name = "quux";
-  clos.pspec = properties[PROP_QUUX];
+  closure.name = "quux";
+  closure.pspec = properties[PROP_QUUX];
 
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "quux", "something new", NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
   /* no change; no notify */
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "quux", "something new", NULL);
-  g_assert (!clos.fired);
+  g_assert (!closure.fired);
 
 
   g_object_unref (obj);
@@ -319,7 +320,7 @@ properties_construct (void)
   gboolean b;
   gchar *s;
 
-  g_test_bug ("630357");
+  g_test_bug ("https://bugzilla.gnome.org/show_bug.cgi?id=630357");
 
   /* more than 16 args triggers a realloc in g_object_new_valist() */
   obj = g_object_new (test_object_get_type (),
@@ -412,6 +413,9 @@ properties_testv_with_valid_properties (void)
   g_assert_cmpstr (g_value_get_string (&values_out[2]), ==, "pigs");
   g_assert_cmpstr (g_value_get_string (&values_out[3]), ==, "fly");
 
+  for (i = 0; i < G_N_ELEMENTS (values_out); i++)
+    g_value_unset (&values_out[i]);
+
   /* Test newv2 && getv */
   g_value_set_string (&(values_in[2]), "Elmo knows");
   g_value_set_string (&(values_in[3]), "where you live");
@@ -425,12 +429,10 @@ properties_testv_with_valid_properties (void)
   g_assert_cmpstr (g_value_get_string (&values_out[2]), ==, "Elmo knows");
   g_assert_cmpstr (g_value_get_string (&values_out[3]), ==, "where you live");
 
-
-  for (i = 0; i < 4; i++)
-    {
-      g_value_unset (&values_in[i]);
-      g_value_unset (&values_out[i]);
-    }
+  for (i = 0; i < G_N_ELEMENTS (values_in); i++)
+    g_value_unset (&values_in[i]);
+  for (i = 0; i < G_N_ELEMENTS (values_out); i++)
+    g_value_unset (&values_out[i]);
 
   g_object_unref (test_obj);
 }
@@ -537,6 +539,51 @@ properties_testv_getv (void)
   g_object_unref (test_obj);
 }
 
+static void
+properties_get_property (void)
+{
+  TestObject *test_obj;
+  struct {
+    const char *name;
+    GType gtype;
+    GValue value;
+  } test_props[] = {
+    { "foo", G_TYPE_INT, G_VALUE_INIT },
+    { "bar", G_TYPE_INVALID, G_VALUE_INIT },
+    { "bar", G_TYPE_STRING, G_VALUE_INIT },
+  };
+  gsize i;
+
+  g_test_summary ("g_object_get_property() accepts uninitialized, "
+                  "initialized, and transformable values");
+
+  for (i = 0; i < G_N_ELEMENTS (test_props); i++)
+    {
+      if (test_props[i].gtype != G_TYPE_INVALID)
+        g_value_init (&(test_props[i].value), test_props[i].gtype);
+    }
+
+  test_obj = (TestObject *) g_object_new_with_properties (test_object_get_type (), 0, NULL, NULL);
+
+  g_test_message ("Test g_object_get_property with an initialized value");
+  g_object_get_property (G_OBJECT (test_obj), test_props[0].name, &(test_props[0].value));
+  g_assert_cmpint (g_value_get_int (&(test_props[0].value)), ==, 42);
+
+  g_test_message ("Test g_object_get_property with an uninitialized value");
+  g_object_get_property (G_OBJECT (test_obj), test_props[1].name, &(test_props[1].value));
+  g_assert_true (g_value_get_boolean (&(test_props[1].value)));
+
+  g_test_message ("Test g_object_get_property with a transformable value");
+  g_object_get_property (G_OBJECT (test_obj), test_props[2].name, &(test_props[2].value));
+  g_assert_true (G_VALUE_HOLDS_STRING (&(test_props[2].value)));
+  g_assert_cmpstr (g_value_get_string (&(test_props[2].value)), ==, "TRUE");
+
+  for (i = 0; i < G_N_ELEMENTS (test_props); i++)
+    g_value_unset (&(test_props[i].value));
+
+  g_object_unref (test_obj);
+}
+
 static void
 properties_testv_notify_queue (void)
 {
@@ -589,12 +636,11 @@ main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
 
-  g_test_bug_base ("http://bugzilla.gnome.org/");
-
   g_test_add_func ("/properties/install", properties_install);
   g_test_add_func ("/properties/notify", properties_notify);
   g_test_add_func ("/properties/notify-queue", properties_notify_queue);
   g_test_add_func ("/properties/construct", properties_construct);
+  g_test_add_func ("/properties/get-property", properties_get_property);
 
   g_test_add_func ("/properties/testv_with_no_properties",
       properties_testv_with_no_properties);
