@@ -37,7 +37,7 @@ static GOptionEntry cmd_entries[] = {
    "Print extra information", NULL},
   {"seconds", 's', 0, G_OPTION_ARG_INT, &test_length,
    "Time to run each test in seconds", NULL},
-  {NULL}
+  G_OPTION_ENTRY_NULL
 };
 
 typedef struct _PerformanceTest PerformanceTest;
@@ -575,8 +575,8 @@ test_type_check_run (PerformanceTest *test,
 		     gpointer _data)
 {
   struct TypeCheckTest *data = _data;
-  volatile GObject *object = data->object;
-  volatile GType type, types[5];
+  GObject *object = data->object;
+  GType type, types[5];
   int i, j;
 
   types[0] = test_iface1_get_type ();
@@ -1015,7 +1015,7 @@ static PerformanceTest tests[] = {
 static PerformanceTest *
 find_test (const char *name)
 {
-  int i;
+  gsize i;
   for (i = 0; i < G_N_ELEMENTS (tests); i++)
     {
       if (strcmp (tests[i].name, name) == 0)
@@ -1051,8 +1051,9 @@ main (int   argc,
     }
   else
     {
-      for (i = 0; i < G_N_ELEMENTS (tests); i++)
-	run_test (&tests[i]);
+      gsize k;
+      for (k = 0; k < G_N_ELEMENTS (tests); k++)
+        run_test (&tests[k]);
     }
 
   return 0;
